@@ -106,7 +106,9 @@ META_ITEM *meta_getitembyextspec(META *m, char *spec, ssize_t specl,
 
 	else if (str_eq(spec, l, pstr("last"))) 
 	    ref.have |= AVR_HAVE_INST, ref.inst = AVR_INST_LAST;
-	else if (str_eq(spec, l, pstr("first")) || str_eq(spec, l, pstr("F")))
+	else if (str_eq(spec, l, pstr("first")) || 
+		 str_eq(spec, l, pstr("f")) ||
+		 str_eq(spec, l, pstr("F")))
 	    ref.have |= AVR_HAVE_INST, ref.inst = AVR_INST_FIRST;
 
 	/* the following are ignored when used in the behaviour language */
@@ -995,7 +997,7 @@ int meta_binmsgtoav(META *m, U_INT32_T *buf, ssize_t len,
 
 	/* Loop through attributes, starting at (char *)msg + 8 */
 	e = buf + ((len + 3) >> 2);
-	for(i = buf + 2; i < e; i += (atrlen + 3) >> 2) {
+	for(i = buf + 2; i > buf && i < e; i += (atrlen + 3) >> 2) {
 
 		/* Get values from attribute header. Warning: netint32 will
 		   evaluate its argument 4 times if little-endian ;o) */
@@ -1005,6 +1007,12 @@ int meta_binmsgtoav(META *m, U_INT32_T *buf, ssize_t len,
 		atrlen = netint32(*i); i++;
 		D1(msg(F_PROC, L_DEBUG, "- got spcnr %ld, vndnr %ld, atrnr %ld, atrlen %ld\n", (long)spcnr, (long)vndnr, (long)atrnr, (long)atrlen));
 
+		if (atrlen > C_MAX_MSGSIZE) { 
+		    msg(F_PROC, L_ERR, "meta_binmsgtoav: Malformed "
+			"pair (length %ld) - ignoring rest\n", (long)atrlen);
+		    break;	    
+		}
+
 		spc = meta_getspcbynr(m, spcnr);
 		if (!spc) { msg(F_PROC, L_NOTICE, "meta_binmsgtoav: Ignoring "
 						  "attribute in "
