@@ -770,6 +770,20 @@ int vm_run(META *m, VM *vm)
 					 R.p + R.l - 16, 16);
 			CHGSTR(&L, s, L.l);
 			break;
+
+		  case OP_SALTENCR:
+			/* Similar to PAPENCR, but uses scheme from RFC 2868
+			 * (tunneling) and RFC 2548 (microsoft VSAs). Always
+			 * generates longer strings, so we always allocate
+			 * a new one. */
+		  	if (R.l <= 16) { meta_freeavdata(&L); L.l = 0; break; }
+			NEWSTR(s, L.l + 18);
+			memcpy(s, L.p, L.l);
+			encrypt_attr_style_1(s, &L.l, R.p, R.l - 16,
+					     R.p + R.l - 16);
+			CHGSTR(&L, s, L.l);
+			break;
+
 		  case OP_PAPDECR:
 			/* Right argument holds secret plus fixed 16-octet
 			 * authenticator; left argument holds a number of 
@@ -943,10 +957,16 @@ int vm_run(META *m, VM *vm)
 			/* Already free right operand's data if any */
 			meta_freeavdata(&R); R.p = 0; R.l = 0;
 
-			/* Go forward through the list */
+			/* Go forward through the list, removing all instances
+			 * of right operand's dictionary item, or all instances
+			 * of items in same space as right operand, if right
+			 * operand's dictionary item has number 'x' (Any) */
+
 			for(av = vm->head[R.ref.list]; av; ) {
 			   avlst = av->next;
-			   if (av->i == R.i) {
+			   if (av->i == R.i ||
+			       (av->i->spc == R.i->spc && 
+				R.i->nr == C_DI_ANY)) {
 
 			   	/* Remove underlying item from list */
 				if (av->next) av->next->prev = av->prev;
